@@ -1,5 +1,6 @@
 /***************************************************************************
  *   Copyright © 2010 Jonathan Thomas <echidnaman@kubuntu.org>             *
+ *   Copyright © 2012 Aleix Pol Gonzalez <aleixpol@blue-systems.com>                *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or         *
  *   modify it under the terms of the GNU General Public License as        *
@@ -20,6 +21,7 @@
 
 #include "ApplicationDetailsWidget.h"
 
+// Qt includes
 #include <QApplication>
 #include <QPropertyAnimation>
 #include <QtCore/QStringBuilder>
@@ -36,6 +38,7 @@
 #include <QtGui/QPushButton>
 #include <QtGui/QVBoxLayout>
 
+// KDE includes
 #include <KDialog>
 #include <KGlobal>
 #include <KHBox>
@@ -53,31 +56,29 @@
 #include <KDebug>
 #include <Nepomuk/KRatingWidget>
 
+// LibQApt includes
 #include <LibQApt/Package>
 
-
+// QZeitgeist includes
 #include "HaveQZeitgeist.h"
-#ifdef HAVE_QZEITGEIST
-#include <QtZeitgeist/DataModel/Event>
-#include <QtZeitgeist/DataModel/TimeRange>
-#include <QtZeitgeist/Log>
-#include <QtZeitgeist/Interpretation>
-#include <QtZeitgeist/Manifestation>
-#include <QtZeitgeist/QtZeitgeist>
-#endif
 
+// Libmuon includes
+#include <Application.h>
+#include <MuonStrings.h>
+#include <ReviewsBackend/Rating.h>
+#include <ReviewsBackend/Review.h>
+#include <ReviewsBackend/ReviewsBackend.h>
+#include <Transaction/TransactionListener.h>
+#include "../../libmuon/mobile/src/mousecursor.h"
+
+// std includes
 #include <math.h>
 
+// Own includes
 #include "AddonsWidget.h"
-#include "Application.h"
 #include "ClickableLabel.h"
+#include "ReviewsWidget/ReviewsWidget.h"
 #include "ScreenShotOverlay.h"
-#include "ReviewsBackend/Rating.h"
-#include "ReviewsBackend/Review.h"
-#include "ReviewsBackend/ReviewsWidget.h"
-#include "ReviewsBackend/ReviewsBackend.h"
-#include "../../libmuon/MuonStrings.h"
-#include "../../libmuon/mobile/src/mousecursor.h"
 
 #define BLUR_RADIUS 15
 
@@ -165,6 +166,7 @@ ApplicationDetailsWidget::ApplicationDetailsWidget(QWidget *parent, ApplicationB
 
     m_actionButton = new QPushButton(actionButtonWidget);
     connect(m_actionButton, SIGNAL(clicked()), this, SLOT(actionButtonClicked()));
+    connect(m_appBackend, SIGNAL(reloadFinished()), this, SLOT(updateActionButton()));
 
     m_progressBar = new QProgressBar(actionButtonWidget);
     m_progressBar->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Preferred);
@@ -259,6 +261,7 @@ ApplicationDetailsWidget::ApplicationDetailsWidget(QWidget *parent, ApplicationB
     detailsGrid->setColumnStretch(1,1);
 
     m_reviewsWidget = new ReviewsWidget(widget);
+    connect(m_reviewsWidget, SIGNAL(fetchPage(int)), this, SLOT(fetchReviews(int)));
 
     QWidget *verticalSpacer = new QWidget(this);
     verticalSpacer->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding);
@@ -286,11 +289,14 @@ ApplicationDetailsWidget::ApplicationDetailsWidget(QWidget *parent, ApplicationB
     layout->addWidget(detailsWidget);
     layout->addWidget(m_reviewsWidget);
     layout->addWidget(verticalSpacer);
+    
+    m_listener = new TransactionListener(this);
+    m_listener->setBackend(m_appBackend);
 
-    connect(m_appBackend, SIGNAL(workerEvent(QApt::WorkerEvent,Transaction*)),
-            this, SLOT(workerEvent(QApt::WorkerEvent,Transaction*)));
-    connect(m_appBackend, SIGNAL(transactionCancelled(Application*)),
-            this, SLOT(transactionCancelled(Application*)));
+    connect(m_listener, SIGNAL(progressChanged()), SLOT(progressChanged()));
+    connect(m_listener, SIGNAL(commentChanged()), SLOT(progressCommentChanged()));
+    connect(m_listener, SIGNAL(running(bool)), SLOT(applicationRunningChanged(bool)));
+    connect(m_listener, SIGNAL(downloading(bool)), SLOT(applicationDownloadingChanged(bool)));
 
     setWidget(widget);
 }
@@ -303,6 +309,7 @@ ApplicationDetailsWidget::~ApplicationDetailsWidget()
 void ApplicationDetailsWidget::setApplication(Application *app)
 {
     m_app = app;
+    m_listener->setApplication(m_app);
 
     // FIXME: Always keep label size at 48x48, and render the largest size
     // we can up to that point. Otherwise some icons will be blurry
@@ -323,8 +330,13 @@ void ApplicationDetailsWidget::setApplication(Application *app)
         m_ratingCountLabel->hide();
     }
 
-    populateZeitgeistInfo();
-
+#ifdef HAVE_QZEITGEIST
+    m_usageLabel->setText(i18ncp("@label The number of times an app has been used",
+                                  "Used one time", "(Used %1 times)", app->usageCount()));
+#else
+    m_usageLabel->hide();
+#endif
+    
     QString menuPathString = app->menuPath();
     if (!menuPathString.isEmpty()) {
         m_menuPathLabel->setText(menuPathString);
@@ -332,16 +344,7 @@ void ApplicationDetailsWidget::setApplication(Application *app)
         m_menuPathWidget->hide();
     }
 
-    if (!app->package()->isInstalled()) {
-        m_statusLabel->setText(MuonStrings::global()->packageStateName(QApt::Package::NotInstalled));
-        m_actionButton->setText(i18nc("@action", "Install"));
-        m_actionButton->setIcon(KIcon("download"));
-        m_actionButton->show();
-    } else {
-        m_statusLabel->setText(MuonStrings::global()->packageStateName(QApt::Package::Installed));
-        m_actionButton->setText(i18nc("@action", "Remove"));
-        m_actionButton->setIcon(KIcon("edit-delete"));
-    }
+    updateActionButton();
 
     m_longDescLabel->setText(app->package()->longDescription());
 
@@ -374,14 +377,7 @@ void ApplicationDetailsWidget::setApplication(Application *app)
                             app->package()->installedVersion());
     }
 
-    if (app->package()->component() == "main" ||
-        app->package()->component() == "universe") {
-        m_license->setText(i18nc("@info license", "Open Source"));
-    } else if (app->package()->component() == "restricted") {
-        m_license->setText(i18nc("@info license", "Proprietary"));
-    } else {
-        m_license->setText(i18nc("@info license", "Unknown"));
-    }
+    m_license->setText(app->license());
 
     if (app->package()->isSupported()) {
         m_support->setText(i18nc("@info Tells how long Canonical, Ltd. will support a package",
@@ -395,196 +391,12 @@ void ApplicationDetailsWidget::setApplication(Application *app)
 
     // Fetch reviews
     connect(reviewsBackend, SIGNAL(reviewsReady(Application*,QList<Review*>)),
-            this, SLOT(populateReviews(Application*,QList<Review*>)));
-    reviewsBackend->fetchReviews(app);
-
-    // Catch already-begun downloads. If the state is something else, we won't
-    // care because we won't handle it
-    QPair<QApt::WorkerEvent, Transaction *> workerState = m_appBackend->workerState();
-    workerEvent(workerState.first, workerState.second);
-
-    foreach (Transaction *transaction, m_appBackend->transactions()) {
-        if (transaction->application() == m_app){
-            showTransactionState(transaction);
-        }
-    }
+	    this, SLOT(populateReviews(Application*,QList<Review*>)));
+    fetchReviews(1);
 
     fetchScreenshot(QApt::Thumbnail);
 }
 
-void ApplicationDetailsWidget::workerEvent(QApt::WorkerEvent event, Transaction *transaction)
-{
-    if (!transaction || !m_appBackend->transactions().contains(transaction) ||
-        m_app != transaction->application()) {
-        return;
-    }
-
-    switch (event) {
-    case QApt::PackageDownloadStarted:
-        m_actionButton->hide();
-        m_cancelButton->show();
-        m_progressBar->show();
-        m_progressBar->setFormat(i18nc("@info:status", "Downloading"));
-        connect(m_appBackend, SIGNAL(progress(Transaction*,int)),
-                this, SLOT(updateProgress(Transaction*,int)));
-        break;
-    case QApt::PackageDownloadFinished:
-        disconnect(m_appBackend, SIGNAL(progress(Transaction*,int)),
-                   this, SLOT(updateProgress(Transaction*,int)));
-        break;
-    case QApt::CommitChangesStarted:
-        m_actionButton->hide();
-        m_cancelButton->hide();
-        m_progressBar->show();
-        m_progressBar->setValue(0);
-        switch(transaction->action()) {
-        case InstallApp:
-            m_progressBar->setFormat(i18nc("@info:status", "Installing"));
-            break;
-        case ChangeAddons:
-            m_progressBar->setFormat(i18nc("@info:status", "Changing Addons"));
-            break;
-        case RemoveApp:
-            m_progressBar->setFormat(i18nc("@info:status", "Removing"));
-            break;
-        default:
-            break;
-        }
-        connect(m_appBackend, SIGNAL(progress(Transaction*,int)),
-                this, SLOT(updateProgress(Transaction*,int)));
-        break;
-    case QApt::CommitChangesFinished:
-        disconnect(m_appBackend, SIGNAL(progress(Transaction*,int)),
-                   this, SLOT(updateProgress(Transaction*,int)));
-        break;
-    default:
-        break;
-    }
-}
-
-void ApplicationDetailsWidget::updateProgress(Transaction *transaction, int percentage)
-{
-    if (m_app == transaction->application()) {
-        m_progressBar->setValue(percentage);
-
-        if (percentage == 100) {
-            m_progressBar->setFormat(i18nc("@info:status Progress text when done", "Done"));
-        }
-    }
-}
-
-void ApplicationDetailsWidget::showTransactionState(Transaction *transaction)
-{
-    m_actionButton->hide();
-    m_progressBar->show();
-    m_progressBar->setValue(0);
-
-    QString text;
-    switch (transaction->state()) {
-    case QueuedState:
-        text = i18nc("@info:status Progress text when waiting", "Waiting");
-        break;
-    case RunningState:
-        switch (transaction->action()) {
-        case InstallApp:
-            text = i18nc("@info:status", "Installing");
-            break;
-        case ChangeAddons:
-            text = i18nc("@info:status", "Changing Addons");
-            break;
-        case RemoveApp:
-            text = i18nc("@info:status", "Removing");
-            break;
-        default:
-            break;
-        }
-        break;
-    case DoneState:
-        text = i18nc("@info:status Progress text when done", "Done");
-        m_progressBar->setValue(100);
-        break;
-    default:
-        break;
-    }
-    m_progressBar->setFormat(text);
-}
-
-void ApplicationDetailsWidget::transactionCancelled(Application *app)
-{
-    if (m_app == app) {
-        m_progressBar->hide();
-        m_cancelButton->hide();
-        m_actionButton->show();
-    }
-}
-
-void ApplicationDetailsWidget::populateZeitgeistInfo()
-{
-#ifdef HAVE_QZEITGEIST
-    if (!m_app->package()->isInstalled()) {
-        m_usageLabel->hide();
-        return;
-    }
-
-    QString desktopFile;
-
-    foreach (const QString &desktop, m_app->package()->installedFilesList().filter(".desktop")) {
-            KService::Ptr service = KService::serviceByDesktopPath(desktop);
-            if (!service) {
-                break;
-            }
-
-            if (service->isApplication() &&
-              !service->noDisplay() &&
-              !service->exec().isEmpty())
-            {
-                desktopFile = desktop.split('/').last();
-                break;
-            }
-    }
-
-    QtZeitgeist::init();
-
-    // Prepare the Zeitgeist query
-    QtZeitgeist::DataModel::EventList eventListTemplate;
-
-    QtZeitgeist::DataModel::Event event1;
-    event1.setActor("application://" + desktopFile);
-    event1.setInterpretation(QtZeitgeist::Interpretation::Event::ZGModifyEvent);
-    event1.setManifestation(QtZeitgeist::Manifestation::Event::ZGUserActivity);
-
-    QtZeitgeist::DataModel::Event event2;
-    event2.setActor("application://" + desktopFile);
-    event2.setInterpretation(QtZeitgeist::Interpretation::Event::ZGCreateEvent);
-    event2.setManifestation(QtZeitgeist::Manifestation::Event::ZGUserActivity);
-
-    eventListTemplate << event1 << event2;
-
-    QtZeitgeist::Log log;
-    QDBusPendingReply<QtZeitgeist::DataModel::EventIdList> reply = log.findEventIds(QtZeitgeist::DataModel::TimeRange::always(),
-                                                                                    eventListTemplate,
-                                                                                    QtZeitgeist::Log::Any,
-                                                                                    0, QtZeitgeist::Log::MostRecentEvents);
-    reply.waitForFinished();
-
-    int usageCount = 0;
-    if (reply.isValid()) {
-        usageCount = reply.value().size();
-    }
-
-    if (!usageCount) {
-        // More likely there are no stats than no usage, so just hide the label and return
-        m_usageLabel->hide();
-        return;
-    }
-
-    m_usageLabel->setText(i18ncp("@label The number of times an app has been used",
-                                  "Used one time", "(Used %1 times)", usageCount));
-#else
-    m_usageLabel->hide();
-#endif // HAVE_QZEITGEIST
-}
-
 void ApplicationDetailsWidget::fetchScreenshot(QApt::ScreenshotType screenshotType)
 {
     if (m_screenshotFile) {
@@ -666,6 +478,9 @@ void ApplicationDetailsWidget::actionButtonClicked()
 void ApplicationDetailsWidget::cancelButtonClicked()
 {
     emit cancelButtonClicked(m_app);
+
+    m_progressBar->hide();
+    m_actionButton->show();
 }
 
 void ApplicationDetailsWidget::populateAddons()
@@ -678,6 +493,16 @@ void ApplicationDetailsWidget::populateAddons()
     }
 }
 
+void ApplicationDetailsWidget::fetchReviews(int page)
+{
+    if (!m_app) {
+	return;
+    }
+
+    m_appBackend->reviewsBackend()->fetchReviews(m_app, page);
+
+}
+
 void ApplicationDetailsWidget::populateReviews(Application *app, const QList<Review *> &reviews)
 {
     if (app != m_app) {
@@ -698,4 +523,42 @@ void ApplicationDetailsWidget::addonsApplyButtonClicked(const QHash<QApt::Packag
     m_progressBar->setFormat(i18nc("@info:status Progress text when waiting", "Waiting"));
 }
 
+void ApplicationDetailsWidget::applicationRunningChanged(bool running)
+{
+    m_actionButton->setVisible(!running);
+    m_progressBar->setVisible(running);
+}
+
+void ApplicationDetailsWidget::applicationDownloadingChanged(bool downloading)
+{
+    m_cancelButton->setVisible(downloading);
+}
+
+void ApplicationDetailsWidget::progressChanged()
+{
+    m_progressBar->setValue(m_listener->progress());
+}
+
+void ApplicationDetailsWidget::progressCommentChanged()
+{
+    m_progressBar->setFormat(m_listener->comment());
+}
+
+void ApplicationDetailsWidget::updateActionButton()
+{
+    if (!m_app)
+        return;
+
+    if (!m_app->package()->isInstalled()) {
+        m_statusLabel->setText(MuonStrings::global()->packageStateName(QApt::Package::NotInstalled));
+        m_actionButton->setText(i18nc("@action", "Install"));
+        m_actionButton->setIcon(KIcon("download"));
+        m_actionButton->show();
+    } else {
+        m_statusLabel->setText(MuonStrings::global()->packageStateName(QApt::Package::Installed));
+        m_actionButton->setText(i18nc("@action", "Remove"));
+        m_actionButton->setIcon(KIcon("edit-delete"));
+    }
+}
+
 #include "ApplicationDetailsWidget.moc"
