@@ -27,7 +27,6 @@
 class QSplitter;
 class QStackedWidget;
 class QToolBox;
-class KAction;
 class KDialog;
 
 class ManagerSettingsDialog;
@@ -38,11 +37,6 @@ class DownloadWidget;
 class CommitWidget;
 class StatusWidget;
 
-namespace QApt
-{
-    class Backend;
-}
-
 /**
  * This class serves as the main window for Muon.  It handles the
  * menus, toolbars, and status bars.
