@@ -138,6 +138,8 @@ PackageWidget::PackageWidget(QWidget *parent)
     connect(m_packageView, SIGNAL(currentPackageChanged(QModelIndex)),
             this, SLOT(packageActivated(QModelIndex)));
     connect(m_packageView, SIGNAL(selectionEmpty()), m_detailsWidget, SLOT(hide()));
+    connect(m_packageView->header(), SIGNAL(sectionClicked(int)),
+            this, SLOT(sectionClicked(int)));
     connect(m_searchEdit, SIGNAL(textChanged(QString)), m_searchTimer, SLOT(start()));
 
     QSplitter *splitter = new QSplitter(this);
@@ -345,6 +347,13 @@ void PackageWidget::setSortedPackages()
     QApplication::restoreOverrideCursor();
 }
 
+void PackageWidget::sectionClicked(int section)
+{
+    if (section == 0) {
+        m_proxyModel->setSortByRelevancy(false);
+    }
+}
+
 void PackageWidget::startSearch()
 {
     if (m_proxyModel->sourceModel()) {
@@ -352,6 +361,13 @@ void PackageWidget::startSearch()
     }
 }
 
+void PackageWidget::invalidateFilter()
+{
+    if (m_proxyModel) {
+        m_proxyModel->invalidate();
+    }
+}
+
 bool PackageWidget::confirmEssentialRemoval()
 {
     QString text = i18nc("@label", "Removing this package may break your system. Are you sure you want to remove it?");
