@@ -40,9 +40,11 @@
 // LibQApt includes
 #include <LibQApt/Backend>
 
+// Libmuon includes
+#include <Application.h>
+#include <ChangesDialog.h>
+
 // Own includes
-#include "../libmuon/ChangesDialog.h"
-#include "../installer/Application.h"
 #include "UpdateModel/UpdateModel.h"
 #include "UpdateModel/UpdateItem.h"
 #include "UpdateModel/UpdateDelegate.h"
@@ -134,6 +136,13 @@ void UpdaterWidget::populateUpdateModel()
 {
     QApt::PackageList upgradeList = m_backend->upgradeablePackages();
 
+    if (upgradeList.isEmpty()) {
+        QApplication::restoreOverrideCursor();
+        m_busyWidget->stop();
+        checkUpToDate();
+        return;
+    }
+
     UpdateItem *securityItem = new UpdateItem(i18nc("@item:inlistbox", "Important Security Updates"),
                                               KIcon("security-medium"));
 
@@ -147,6 +156,8 @@ void UpdaterWidget::populateUpdateModel()
     QStringList fileList = appDir.entryList(QDir::Files);
 
     foreach(const QString &fileName, fileList) {
+        if (fileName.endsWith(QLatin1String(".menu"))) // Skip non-desktop files that can slip in
+            continue;
         Application *app = new Application("/usr/share/app-install/desktop/" + fileName, m_backend);
         QApt::Package *package = app->package();
         if (!package || !upgradeList.contains(package)) {
@@ -241,8 +252,7 @@ void UpdaterWidget::checkApps(QList<Application *> apps, bool checked)
         list << app->package();
     }
 
-    QApt::Package::State action;
-    checked ?  action = QApt::Package::ToInstall : action = QApt::Package::ToKeep;
+    QApt::Package::State action = checked ? QApt::Package::ToInstall : QApt::Package::ToKeep;
 
     if (list.size() > 1) {
         QApplication::setOverrideCursor(Qt::WaitCursor);
@@ -287,7 +297,7 @@ void UpdaterWidget::selectionChanged(const QItemSelection &selected,
     }
 
     Application *app = m_updateModel->itemFromIndex(indexes.first())->app();
-    app ? package = app->package() : package = 0;
+    package = app ? app->package() : 0;
 
     emit packageChanged(package);
 }
